;; Copyright 2009, 2010, 2015 Kevin Ryde.

;; This file is part of accjournal.
;;
;; accjournal is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; accjournal is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with accjournal; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; Generate autoloads for emacsen-startup (which is 50accjournal.el).

(require 'cl)

(defadvice make-autoload (after my-zap-docstring disable)
  "Zap docstrings in `autoload' forms per `my-zap-autoload-docstring'."
  (setq ad-return-value (my-zap-autoload-docstring ad-return-value)))

(defun my-zap-autoload-docstring (form)
  "If FORM is an (autoload ...) return it with docstring zapped.
Otherwise return FORM unchanged."

  ;;  0        1     2      3           4                5
  ;; (autoload 'func "file" "docstring" interactive-flag type)
  ;;
  (when (and (eq 'autoload (car-safe form))
             (stringp (nth 3 form)))
    (setq form (copy-sequence form))
    (setf (nth 3 form) nil))

  (my-list-remove-trailing-nils form))

;; (my-zap-autoload-docstring nil)
;; (my-zap-autoload-docstring '(autoload 'func "file" "docstring" interactive-flag))
;; (my-zap-autoload-docstring '(autoload 'func "file" "docstring" nil nil))
;; (my-zap-autoload-docstring '(autoload 'func "file"))

(defun my-list-remove-trailing-nils (lst)
  "Remove trailing nil elements from list LST."
  (while (and (consp lst)
              (eq nil (car (last lst))))
    (setq lst (butlast lst 1)))
  lst)

(defun my-ad-advice-enabled-p (func class name)
  "Return non-nil if FUNC advice CLASS and NAME is enabled."
  (let ((advice (ad-find-advice func class name)))
    (and advice
         (ad-advice-enabled advice))))

(defmacro my-with-ad-enable-advice (func class name &rest body)
  "Evaluate BODY with FUNC advice CLASS and NAME temporarily enabled.
And `unwind-protect' ensures the advice is re-disabled (if it was
disabled to start with) no matter what BODY does.

In the current implementation CLASS must be a specific class,
like `before' or `around', it can't be `any'."

  (let ((oldsym (make-symbol "old-enable")))
    `(let ((,oldsym (my-ad-advice-enabled-p ,func ,class ,name)))
       (unwind-protect
           (progn
             (unless ,oldsym
               (ad-enable-advice ,func ,class ,name)
               (ad-activate ,func))
             ,@body)
         (unless ,oldsym
           (ad-disable-advice ,func ,class ,name)
           (ad-activate ,func))))))
(put 'my-with-ad-enable-advice 'lisp-indent-function 3)

;;
;; (progn (ad-enable-advice 'make-autoload 'after 'my-zap-docstring) (ad-activate 'make-autoload))
;; (progn (ad-disable-advice 'make-autoload 'after 'my-zap-docstring) (ad-activate 'make-autoload))
;; (make-autoload '(defun foo () "docstring" 123) "foo.el")
;; (my-with-ad-enable-advice 'make-autoload 'after 'my-zap-docstring (make-autoload '(defun foo () "docstring" 123) "foo.el"))
;;
;; (ad-find-advice 'make-autoload 'after 'my-zap-docstring)
;; (my-ad-advice-enabled-p 'make-autoload 'after 'my-zap-docstring)
;;
;; (my-with-ad-enable-advice 'make-autoload 'after 'my-zap-docstring (my-ad-advice-enabled-p 'make-autoload 'after 'my-zap-docstring))
;;

(defun my-zap-blank-lines ()
  "Zap blank lines between autoload forms."
  (save-excursion
    (while (search-forward ")\n\n(" nil t)
      (replace-match ")\n(" t t))))

(defun my-generate-file-autoloads (filename)
  "Like `generate-autoloads' but don't bother with docstrings."
  (let ((print-quoted t))
    (my-with-ad-enable-advice 'make-autoload 'after 'my-zap-docstring
      (save-excursion
        (generate-file-autoloads filename))
      (my-zap-blank-lines))))

;; some nonsense in emacs24 forces the generate to be in a file-visiting
;; buffer, and in the same directory as accjournal.el or else the autoloads
;; include a path "../accjournal"
(insert (progn
          (find-file "../my-generate-autoloads.out.el")
          (erase-buffer)
          (my-generate-file-autoloads "accjournal.el")
          (prog1 (buffer-string)
            (set-buffer-modified-p nil)
            (kill-buffer (current-buffer)))))
